--- The vehicle animation module
-- @author Oskar Eisemuth
-- @copyright 2019
-- @module commonapi.vehanimations
local vehanimations = {
	_callbacks = {},
	dotrace = false,
}

local _utils = commonapi.utils
local _log = _utils.logPrep("commonapi.vehanimations")

--- Adds a event for callback
-- @tparam string mdl modelname
-- @tparam function fn callback function
function vehanimations.add(mdl, fn)
	vehanimations._callbacks[mdl] = fn
end

function vehanimations.traceOn()
	vehanimations.dotrace = true
	_log("Tracing for Animations enabled", vehanimations.dotrace)
end



function vehanimations._activate()
	if (commonapi._native.setVehicleAnimationCallback ~= nil) then
        commonapi._native.setVehicleAnimationCallback(vehanimations._vehiclefireevent)
        return true
	end
	return false
end

function vehanimations._defaultHandler(data, index)
	local veh = data.vehicles[index]
	if (veh.opennum ~= nil) then
		veh.event = veh.event .."_".. veh.opennum
	else
		veh.event = veh.event .."_".. math.random(1, 20)
	end
end

function vehanimations._vehiclefireevent(data)
	if (vehanimations.dotrace) then
		_log("vehanimations._vehiclefireevent")
		--commonapi.dmp(game.interface.getEntity(data.id))
		commonapi.dmp(data)
	end
	if (data.vehicles == nil) then
		return {}
	end
	for i=1, #data.vehicles do
		local cb = vehanimations._callbacks[data.vehicles[i].filename] or nil
		if cb ~= nil then
			if (vehanimations.dotrace) then
				_log("vehanimations._vehiclefireevent using callback for ", i,  data.vehicles[i].filename)
			end
			cb(data, i)
--		else
--			vehanimations._defaultHandler(data, i)
		end
	end
	return data
end

function vehanimations.getLineName(data)
	if (data.line ~= nil) then
		if (data._linedata == nil) then
			data._linedata = game.interface.getEntity(data.line)
		end
		if (data._linedata ~= nil) then
			return data._linedata.name
		end
	end

	return "Unknown"
end

return vehanimations
