local ModCommonListWindow = {}

local _log = commonapi.utils.logPrep("commonapi2.ui.ModCommonListWindow")
local _utils = commonapi.utils
local _ = _utils._t
local ui = commonapi.ui;


--commonapi._reloadModule("_modmm")
local modmm = commonapi._modmm;

ModCommonListWindow = ui.Window:newClass()
ModCommonListWindow.caption = "ModCommonListWindow"

function ModCommonListWindow:genModUIList()
	local uilist = {}
	self.uilist = uilist

	if (self._modmmrepos == nil) then
		self._modmmrepos = modmm.getRepoDataObj()
		self._modmmrepos:reload()
	end

	return uilist
end


function ModCommonListWindow:genRepoDownloadList(uilist, repoentries)
	for mu, repoentry in ipairs( repoentries ) do

		local caption = repoentry.name..' - ('..table.concat(repoentry.version, ".")..')  via '..repoentry.repodata.name
		if (repoentry.utc_changed ~= nil) then
			caption = caption .. '  ' .. os.date('%Y-%m-%d', repoentry.utc_changed)
		end

		uilist[#uilist+1] = {
			typ = "_Selectable",
			id = repoentry.modid..'__repo__'..mu,
			caption = caption,
			data = {
				modid = repoentry.modidid,
				repoentry = repoentry,
			},
			onclick = function(item, win)
				ui.msgbox("DoModUpgrade",
					_("Install ${modname}?") % {
						modname = repoentry.name..' - ('..table.concat(repoentry.version, ".")..')'
					}
					, "Mod upgrade", {
						{ caption = _("Cancel") }, { caption = _("Install or Upgrade"), fn = function()
							local ok, errormsg = modmm.downloadAndInstallMod(item.data.repoentry)
							if (ok == false) then
								ui.msgbox('ModListWindow_downloadAndInstallMod', "Fehler beim Download/Update von:"..repoentry.modid.." \n"..errormsg, "Update")
							end
							win:clearCache()
						end
					},
				})
			end,
		}
	end
end


function ModCommonListWindow:genUpgradeList(uilist, entry)
	for mu, repoentry in ipairs( entry.repomods ) do

		local caption = repoentry.name..' - ('..table.concat(repoentry.version, ".")..')    via '..repoentry.repodata.name
		if (repoentry.utc_changed ~= nil) then
			caption = caption .. '  ' .. os.date('%Y-%m-%d', repoentry.utc_changed)
		end
		uilist[#uilist+1] = {
			typ = "_Selectable",
			id = entry.id..'__repo__'..mu,
			caption = caption,
			data = {
				modid = entry.id,
				repoentry = repoentry,
			},
			onclick = function(item, win)
				--entry.downloadfull = repodata.download_prefix_url..''..entry.download
				--entry.entryurlfull = repodata.entry_prefix_url..''..entry.entryurl
				--commonapi._native.utils.openurl(item.data.repodata.download_prefix_url..item.data.repoentry.download)
				ui.msgbox("DoModUpgrade",
					_("Install ${modname}?") % {
						modname = repoentry.name..' - ('..table.concat(repoentry.version, ".")..')'
					}
					, "Mod upgrade", {
						{ caption = _("Cancel") }, { caption = _("Upgrade"), fn = function()
							local ok, errormsg = modmm.downloadAndInstallMod(item.data.repoentry)
							if (ok == false) then
								ui.msgbox('ModListWindow_downloadAndInstallMod', "Fehler beim Download/Update von:"..entry.id.." \n"..errormsg, "Update")
							end
							win:clearCache()
						end
					},
				})
			end,
		}
	end
end


function ModCommonListWindow:genModMMDownloadLink(uilist, repoentry)
	local caption = repoentry.name..' - ('..table.concat(repoentry.version, ".")..')    via '..repoentry.repodata.name
	uilist[#uilist+1] = {
		typ = "Button",
		id = repoentry.modid..'__modmmdownload__',
		caption = caption,
		data = {
			repoentry = repoentry,
		},
		onclick = function(item, win)
			--entry.downloadfull = repodata.download_prefix_url..''..entry.download
			--entry.entryurlfull = repodata.entry_prefix_url..''..entry.entryurl
			--commonapi._native.utils.openurl(item.data.repodata.download_prefix_url..item.data.repoentry.download)
			ui.msgbox("DoModUpgrade",
				_("Install ${modname}?") % {
					modname = repoentry.name..' - ('..table.concat(repoentry.version, ".")..')'
				}
				, "Mod upgrade", {
					{ caption = _("Cancel") }, { caption = _("Upgrade or Downlad"), fn = function()
						local ok, errormsg = modmm.downloadAndInstallMod(item.data.repoentry)
						if (ok == false) then
							ui.msgbox('ModListWindow_downloadAndInstallMod', "Fehler beim Download/Update von:"..caption.." \n"..errormsg, "Update")
						end
						win:clearCache()
					end
				},
			})
		end,
	}
end

function ModCommonListWindow:genDepList(uilist, modentry)
	local requiredMods = modentry.data.info.requiredMods or {}

	for m, entry in pairs( requiredMods ) do

		local links = commonapi.mods._checkModDepends_createLinks(entry)

		local uimodid = {
			typ = "Label",
			caption = ""..(entry.modId or "No modId").." , "..(entry.steamId or "No SteamId")
		}
		uilist[#uilist+1] = uimodid
		local uiversion = {
			typ = "Label",
			caption = "\tMinorVersion: >="..(entry.minMinorVersion or 0)
		}
		uilist[#uilist+1] = uiversion

		if (entry.modId ~= nil and self._modmmrepos ~= nil) then
			local repoentries = self._modmmrepos:findEntriesByModId(entry.modId)
			if (repoentries ~= nil and #repoentries > 0) then
				ModCommonListWindow.genModMMDownloadLink(self, uilist, repoentries[1])
			end
		end

		for urltype, url in pairs( links ) do
			local urlentry = {
				typ = "Button",
				caption = urltype,
				sameline = true,
				id = ""..m..''..urltype,
				data = {
					myurl = url,
				},
				onclick = function(item, win)
					commonapi._native.utils.openurl(item.data.myurl)
				end
			}
			uilist[#uilist+1] = urlentry
		end
	end
end


function ModCommonListWindow:genErrorList(uilist, modentry)
	local mid = modentry.id..'__errors__'
	for i, error in ipairs( modentry.errors ) do
		local errstring = "?"
		if (error.type == "modmissing") then
			errstring = _("requires ${modId}") % {
				modId = (error.data.modId or error.data.steamId) or "?"
			}
		elseif (error.type == "modorder") then
			errstring = _("should be loaded after ${modname}") % {
				modname = error.mod.name
			}
			--errstring = "\t  ".. error.modname .. "should be loaded first"
		elseif (error.type == "modversion") then
			errstring = _("requires ${modname} ${version}") % {
				modname = error.mod.name,
				version = tostring(error.mod.version[1] or 0) .. "." .. error.data.minMinorVersion
			}
		elseif (error.type == "modinactive") then
			errstring = _("requires ${modname}, please activate") % {
				modname = error.mod.name,
			}
		else
			errstring = error.type
		end
		local uientry = {
			typ = "Label",
			caption = "\t"..errstring
		}
		uilist[#uilist+1] = uientry

		if (self._modmmrepos ~= nil and error.type == "modmissing" or error.type == "modversion") then
			if (error.data.modId ~= nil) then
				local repoentries = self._modmmrepos:findEntriesByModId(error.data.modId)
				if (repoentries ~= nil and #repoentries > 0) then
					ModCommonListWindow.genModMMDownloadLink(self, uilist, repoentries[1])
				end
			end

			if (error.links ~= nil) then
				for urltype, url in pairs( error.links ) do
					local urlentry = {
						typ = "Button",
						caption = urltype,
						sameline = true,
						id = mid..''..urltype,
						myurl = url,
						onclick = function(item, win)
							commonapi._native.utils.openurl(item.myurl)
						end
					}
					uilist[#uilist+1] = urlentry
				end
			end
		end
	end
end

return ModCommonListWindow