local InspectWindow = {}

local ui = require("commonapi2.ui2.tpfui")
local inspectlua = require("commonapi2.extlib.inspect")

InspectWindow = ui.Window:new()

InspectWindow.id =  "InspectWindow"
InspectWindow.caption = "Inspector"
InspectWindow.uilist = {}

InspectWindow.height = 500
InspectWindow.width = 500


local secondaryui = require "commonapi2.ui2.secondaryui"

local function EventTraceOutputCallback(idsource, id, name, param)
	--EventTraceOutput
	local eleid = "XCoTPF2.InspectWindow.EventTraceScrollRegion"
	local ele = api.gui.util.getById(eleid)
	if (ele ~= nil) then

		local layout = ele:getContent():getLayout()
		if (layout) then
			local itemtext = idsource.." "..id.."  "..name.. "\n"..inspectlua(param, { metatables = 0 })
			local item = api.gui.comp.TextView.new(itemtext)
			layout:addItem(item, false)

			if (layout:getNumItems() > 10) then
				local tempitem = layout:getItem(0)
				layout:removeItem(tempitem)
				tempitem:destroy()
			end
		--	layout:ensureVisible(item)
		end
	end
end


local function EventTraceOutputStart()
	secondaryui.traceGUICallback = EventTraceOutputCallback
end

local function EventTraceOutputStop()
	secondaryui.traceGUICallback = nil
end



function InspectWindow.create()
	local o
	o = InspectWindow:new()
	o:createUIList()
	o:open()

	return o
end

function InspectWindow:refresh()
	self:createUIList()
end



function InspectWindow:createUIList()
	local uilist = {}
	local uitabs = {}

	uitabs[#uitabs+1] = {
		typ = "TabItem",
		id = "EntityTab",
		caption = "Entity",
		uilist = self:createUIListTabEntity()
	}

	uitabs[#uitabs+1] = {
		typ = "TabItem",
		id = "EventsTab",
		caption = "Events",
		uilist = self:createUIListTabEvents()
	}


	uilist[#uilist+1] = {
		typ = "TabBar",
		id = "InspectorTabs",
		uilist = uitabs,
	}
	self.uilist = uilist
end

function InspectWindow:createUIListTabEntity()
	local uilist = {}

	local entityId = tonumber(self.params.EntityId)

	uilist[#uilist+1] = {
		typ = "TextInput",
		id = "EntityId",
		changeonenter = true,
		onchange = function(item, win)
			win:refresh()
		end
	}

	local componentTypes = getmetatable(api.type.ComponentType).__index
	local componentTypesKeys = {}

	for k in pairs(componentTypes) do
		componentTypesKeys[#componentTypesKeys+1] = k
	end
	table.sort(componentTypesKeys)

	local uilistregion = {}


	if (entityId ~= nil and entityId > 0) then
	
		uilist[#uilist+1] = {
			typ = "Label",
			caption = "Entity ID: "..entityId
		}

		uilist[#uilist+1] = {
			typ = "ScrollRegion",
			id = "ScrollRegion",
			border = false,
			size = {
				0, -30
			},
			uilist = uilistregion
		}
		
		if (api.engine.entityExists(entityId)) then
			

			for _, componentName in ipairs(componentTypesKeys) do
				local componentTypeId = api.type.ComponentType[componentName]
				local componentData = api.engine.getComponent(entityId, componentTypeId)

				if (componentData ~= nil) then

					uilistregion[#uilistregion+1] = {
						typ = "Separator"
					}

					uilistregion[#uilistregion+1] = {
						typ = "Label",
						caption = componentName
					}

					uilistregion[#uilistregion+1] = {
						typ = "Label",
						caption = inspectlua(componentData),
						id = "data_"..componentName
					}

				end
			end
		end
	else
		uilist[#uilist+1] = {
			typ = "Label",
			caption = "Entity ID: <not found>"
		}
	end

	uilist[#uilist+1] = {
		typ = "Separator"
	}
	uilist[#uilist+1] = {
		typ = "Button",
		id = "Refresh",
		caption = self._("Refresh"),
		onclick = function(item, win)
			win:refresh()
		end
	}
	--self.uilist = uilist
	return uilist
end


function InspectWindow:createUIListTabEvents()
	local uilist = {}
	local uilistregion = {}


	uilist[#uilist+1] = {
		typ = "ScrollRegion",
		id = "EventTraceScrollRegion",
		border = true,
		size = {
			0, -30
		},
		uilist = uilistregion
	}

	uilist[#uilist+1] = {
		typ = "Separator"
	}

	local uilistbuttons = {}
	uilist[#uilist+1] = {
		typ = "LayoutHorizontal",
		id = "EventLayoutHorizontal",
		uilist = uilistbuttons
	}


	uilistbuttons[#uilistbuttons+1] = {
		typ = "Button",
		id = "EventTraceStart",
		caption = self._("Start"),
		onclick = function(item, win)
			EventTraceOutputStart()
		end
	}

	uilistbuttons[#uilistbuttons+1] = {
		typ = "Button",
		id = "EventTraceStop",
		caption = self._("Stop"),
		onclick = function(item, win)
			EventTraceOutputStop()
		end
	}

	uilistbuttons[#uilistbuttons+1] = {
		typ = "Button",
		id = "ClearEventTrace",
		caption = self._("Clear"),
		onclick = function(item, win)
			local eleid = win._ugidprefix.."EventTraceScrollRegion"
			local ele = api.gui.util.getById(eleid)
			if (ele ~= nil) then
				local layout = ele:getContent():getLayout()
				local numitems = layout:getNumItems()
				if (numitems > 0) then
					for i = numitems, 1, -1 do
						local tmp = layout:getItem(i-1)
						layout:removeItem(tmp)
						tmp:destroy()
					end
				end
			end
		end
	}
	return uilist
end

return InspectWindow